#! /bin/bash
# Last edited on 2010-05-07 12:09:41 by stolfilocal

dataset="$1"; shift
methods=( "$@" )

tmp="/tmp/$$"
gplfile="${tmp}.gpl"
epsfile="${tmp}.eps"

# Point types (should be enough):
voidpts=( 4 6 8 12 ) 
fullpts=( 5 7 9 13 ) 

# Gather datapoints and collect one datapoint for each method:
maxsizes=()
maxbytes=()
powers=()
linetypes=()
pointtypes=()
nvoid=0
nfull=0
imth=0
for mt in ${methods[@]} ; do 
  mname=${mt%.*}
  mtag=${mt#*.}
  if [[ "/${mname}" =~ "/mscl" ]]; then
    pwr="1.0"; ltp="1"; ptp="${fullpts[${nfull}]}"; nfull=$(( ${nfull} + 1 )); fact=0
  elif [[ "/${mname}" =~ "/mgrf" ]]; then
    pwr="1.0"; ltp="1"; ptp="${fullpts[${nfull}]}"; nfull=$(( ${nfull} + 1 )); fact=0
  else
    pwr="1.15"; ltp="3"; ptp="${voidpts[${nvoid}]}"; nvoid=$(( ${nvoid} + 1 )); fact=1
  fi
  datafile="${tmp}-${mname}.dat"
  cat data/${dataset}-${mname}-*-CST.txt \
    | sort -k1,1n \
    | gawk -v fact=${fact} \
        ' /^ *[0-9]/ {
            nP = $1; tm = $2; nA = $3; nL = $4; nU = $5;
            mb = (fact ? 12*nA + 16*nU : 65*nP)/1000000.0;
            print nP, mb;
          }
        ' \
    > ${datafile}
  last=( `tail -1 ${datafile}` )
  mxsz="${last[0]}";
  mxmb="${last[1]}"
  echo "mname = ${mname}  mtag = ${mtag}  pwr = ${pwr}  ltp = ${ltp}  ptp=${ptp}  mxsz = ${mxsz}  mxmb = ${mxmb}" 1>&2;
  maxsizes[${imth}]="${mxsz}"
  maxbytes[${imth}]="${mxmb}"
  powers[${imth}]="${pwr}"
  linetypes[${imth}]="${ltp}"
  pointtypes[${imth}]="${ptp}"
  imth=$(( ${imth} + 1 ))
done

# Create plotfile
cat > ${gplfile} <<EOF
  set term postscript eps monochrome "NonzelsRoman" 24 \
    dashlength 2.0 linewidth 2.0 \
    size 4.0,2.5 
  set pointsize 1.5
  set output "${epsfile}" 
  set xrange [(64*64/2):(512*512*2)]
  set logscale x
  set yrange [0.1/1.015:1000.0*1.05]
  set title "${dataset} (MB)" offset 0,-2.25
  set key left reverse Left samplen 1.5
  set xtics ( \
    "32x32" (32*32), \
    "64x64" (64*64), \
    "128x128" (128*128), \
    "256x256" (256*256), \
    "512x512" (512*512) \
  )
  set logscale y
EOF

imth=0
sep=""
printf 'plot' >> ${gplfile}
for mt in ${methods[@]} ; do 
  mname=${mt%.*}
  mtag=${mt#*.}
  echo "mname = ${mname}  mtag = ${mtag}" 1>&2;
  datafile="${tmp}-${mname}.dat"
  mxsz="${maxsizes[${imth}]}"
  mxmb="${maxbytes[${imth}]}"
  pwr="${powers[${imth}]}"
  ltp="${linetypes[${imth}]}"
  ptp="${pointtypes[${imth}]}"
  printf "${sep}"' \\\n' >> ${gplfile}
  printf '  '"${mxmb}"'*((x+0.0)/'"${mxsz}"')**'"${pwr}"' notitle with lines' >> ${gplfile}
  printf ' lt '"${ltp}"', \\\n' >> ${gplfile}
  printf '  "'"${datafile}"'" using 1:2 title "'"${mtag}"'" with points pt '"${ptp}" >> ${gplfile}
  imth=$(( ${imth} + 1 ))
  sep=","
done
  
printf '\n' >> ${gplfile}
printf 'quit\n' >> ${gplfile}

gnuplot < ${gplfile}

# ( gv ${epsfile}; echo "ta-da." 1>&2; )

cat ${epsfile}

rm -f ${tmp}.* ${tmp}-*.*
